@@ -1,5 +1,5 @@
 /*
- *  SPDX-FileCopyrightText: 2017 by Marco Martin <mart@kde.org>
+ *  SPDX-FileCopyrightText: 2017 Marco Martin <mart@kde.org>
  *
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
@@ -69,8 +69,8 @@ class KIRIGAMIPLATFORM_EXPORT PlatformTheme : public QObject
 
     /**
      * Foreground color for disabled areas, usually a mid-gray
-     * @note Depending on the implementation, the color used for this property may not be 
-     *       based on the disabled palette. For example, for the Plasma implementation, 
+     * @note Depending on the implementation, the color used for this property may not be
+     *       based on the disabled palette. For example, for the Plasma implementation,
      *       "Inactive Text Color" of the active palette is used.
      */
     Q_PROPERTY(QColor disabledTextColor READ disabledTextColor WRITE setCustomDisabledTextColor RESET setCustomDisabledTextColor NOTIFY colorsChanged FINAL)
@@ -355,13 +355,65 @@ protected:
 private:
     KIRIGAMIPLATFORM_NO_EXPORT void update();
     KIRIGAMIPLATFORM_NO_EXPORT void updateChildren(QObject *item);
-    KIRIGAMIPLATFORM_NO_EXPORT void emitSignals();
-    KIRIGAMIPLATFORM_NO_EXPORT void emitColorChanged();
     KIRIGAMIPLATFORM_NO_EXPORT QObject *determineParent(QObject *object);
+    KIRIGAMIPLATFORM_NO_EXPORT void emitSignalsForChanges(int changes);
 
     PlatformThemePrivate *d;
     friend class PlatformThemePrivate;
     friend class PlatformThemeData;
+    friend class PlatformThemeChangeTracker;
+};
+
+/**
+ * A class that tracks changes to PlatformTheme properties and emits signals at the right moment.
+ *
+ * This should be used by PlatformTheme implementations to ensure that multiple
+ * changes to a PlatformTheme's properties do not emit multiple change signals,
+ * instead batching all of them into a single signal emission. This then ensures
+ * things making use of PlatformTheme aren't needlessly redrawn or redrawn in a
+ * partially changed state.
+ *
+ * @since 6.7
+ *
+ */
+class KIRIGAMIPLATFORM_EXPORT PlatformThemeChangeTracker
+{
+public:
+    /**
+     * Flags used to indicate changes made to certain properties.
+     */
+    enum class PropertyChange : uint8_t {
+        None = 0,
+        ColorSet = 1 << 0,
+        ColorGroup = 1 << 1,
+        Color = 1 << 2,
+        Palette = 1 << 3,
+        Font = 1 << 4,
+        Data = 1 << 5,
+        All = ColorSet | ColorGroup | Color | Palette | Font | Data,
+    };
+    Q_DECLARE_FLAGS(PropertyChanges, PropertyChange)
+
+    PlatformThemeChangeTracker(PlatformTheme *theme, PropertyChanges changes = PropertyChange::None);
+    ~PlatformThemeChangeTracker();
+
+    void markDirty(PropertyChanges changes);
+
+private:
+    PlatformTheme *m_theme;
+
+    // Per-PlatformTheme data that we need for PlatformThemeChangeBlocker.
+    // We don't want to store this in PlatformTheme since that would increase the
+    // size of every instance of PlatformTheme while it's only used when we want to
+    // block property change signal emissions. So instead we store it in a separate
+    // hash using the PlatformTheme as key.
+    struct Data {
+        PropertyChanges changes;
+    };
+
+    std::shared_ptr<Data> m_data;
+
+    inline static QHash<PlatformTheme *, std::weak_ptr<Data>> s_blockedChanges;
 };
 
 namespace PlatformThemeEvents
@@ -417,4 +469,6 @@ KIRIGAMIPLATFORM_EXPORT QEvent::Type FontChangedEvent::type;
 }
 } // namespace Kirigami
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(Kirigami::Platform::PlatformThemeChangeTracker::PropertyChanges)
+
 #endif // PLATFORMTHEME_H
